package java.awt;

import sun.java2d.DisposerRecord;
import java.lang.reflect.InvocationTargetException;
import java.awt.image.WritableRaster;
import sun.awt.image.SunWritableRaster;
import java.awt.image.Raster;
import java.awt.image.DataBufferInt;
import java.awt.image.BufferedImage;
import sun.security.util.SecurityConstants;
import java.awt.event.InputEvent;
import sun.awt.SunToolkit;
import sun.java2d.Disposer;
import sun.awt.ComponentFactory;
import java.awt.image.DirectColorModel;
import java.awt.peer.RobotPeer;

public class Robot
{
    private static final int MAX_DELAY = 60000;
    private RobotPeer peer;
    private boolean isAutoWaitForIdle;
    private int autoDelay;
    private static int LEGAL_BUTTON_MASK;
    private DirectColorModel screenCapCM;
    private transient Object anchor;
    private transient Robot.RobotDisposer disposer;
    
    public Robot() throws AWTException {
        this.isAutoWaitForIdle = false;
        this.autoDelay = 0;
        this.screenCapCM = null;
        this.anchor = new Object();
        if (GraphicsEnvironment.isHeadless()) {
            throw new AWTException("headless environment");
        }
        this.init(GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice());
        return;
    }
    
    public Robot(GraphicsDevice graphicsDevice) throws AWTException {
        this.isAutoWaitForIdle = false;
        this.autoDelay = 0;
        this.screenCapCM = null;
        this.anchor = new Object();
        this.checkIsScreenDevice(graphicsDevice);
        this.init(graphicsDevice);
    }
    
    private void init(GraphicsDevice graphicsDevice) throws AWTException {
        this.checkRobotAllowed();
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if ((defaultToolkit instanceof ComponentFactory)) {
            this.peer = ((ComponentFactory)defaultToolkit).createRobot(this, graphicsDevice);
            this.disposer = new Robot.RobotDisposer(this.peer);
            Disposer.addRecord(this.anchor, this.disposer);
        }
        initLegalButtonMask();
    }
    
    private static synchronized void initLegalButtonMask() {
        if (Robot.LEGAL_BUTTON_MASK != 0) {
            return;
        }
        int n = 0;
        if (Toolkit.getDefaultToolkit().areExtraMouseButtonsEnabled() && Toolkit.getDefaultToolkit() instanceof SunToolkit) {
            for (int numberOfButtons = ((SunToolkit)(SunToolkit)Toolkit.getDefaultToolkit()).getNumberOfButtons(), i = 0; i < numberOfButtons; ++i) {
                n |= InputEvent.getMaskForButton(i + 1);
            }
        }
        Robot.LEGAL_BUTTON_MASK = (n | 0x1C1C);
        return;
    }
    
    private void checkRobotAllowed() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.AWT.CREATE_ROBOT_PERMISSION);
        }
    }
    
    private void checkIsScreenDevice(GraphicsDevice graphicsDevice) {
        if (graphicsDevice == null || graphicsDevice.getType() != 0) {
            throw new IllegalArgumentException("not a valid screen device");
        }
        return;
    }
    
    public synchronized void mouseMove(int n, int n2) {
        this.peer.mouseMove(n, n2);
        this.afterEvent();
    }
    
    public synchronized void mousePress(int n) {
        this.checkButtonsArgument(n);
        this.peer.mousePress(n);
        this.afterEvent();
    }
    
    public synchronized void mouseRelease(int n) {
        this.checkButtonsArgument(n);
        this.peer.mouseRelease(n);
        this.afterEvent();
    }
    
    private void checkButtonsArgument(int n) {
        if ((n | Robot.LEGAL_BUTTON_MASK) != Robot.LEGAL_BUTTON_MASK) {
            throw new IllegalArgumentException("Invalid combination of button flags");
        }
        return;
    }
    
    public synchronized void mouseWheel(int n) {
        this.peer.mouseWheel(n);
        this.afterEvent();
    }
    
    public synchronized void keyPress(int n) {
        this.checkKeycodeArgument(n);
        this.peer.keyPress(n);
        this.afterEvent();
    }
    
    public synchronized void keyRelease(int n) {
        this.checkKeycodeArgument(n);
        this.peer.keyRelease(n);
        this.afterEvent();
    }
    
    private void checkKeycodeArgument(int n) {
        if (n == 0) {
            throw new IllegalArgumentException("Invalid key code");
        }
        return;
    }
    
    public synchronized Color getPixelColor(int n, int n2) {
        checkScreenCaptureAllowed();
        return new Color(this.peer.getRGBPixel(n, n2));
    }
    
    public synchronized BufferedImage createScreenCapture(Rectangle rectangle) {
        checkScreenCaptureAllowed();
        checkValidRect(rectangle);
        if (this.screenCapCM == null) {
            this.screenCapCM = new DirectColorModel(24, 16711680, 65280, 255);
        }
        Toolkit.getDefaultToolkit().sync();
        int[] bandMasks = new int[3];
        int[] rgbPixels = this.peer.getRGBPixels(rectangle);
        DataBufferInt dataBufferInt = new DataBufferInt(rgbPixels, rgbPixels.length);
        bandMasks[0] = this.screenCapCM.getRedMask();
        bandMasks[1] = this.screenCapCM.getGreenMask();
        bandMasks[2] = this.screenCapCM.getBlueMask();
        WritableRaster packedRaster = Raster.createPackedRaster(dataBufferInt, rectangle.width, rectangle.height, rectangle.width, bandMasks, null);
        SunWritableRaster.makeTrackable(dataBufferInt);
        return new BufferedImage(this.screenCapCM, packedRaster, false, null);
    }
    
    private static void checkValidRect(Rectangle rectangle) {
        if (rectangle.width <= 0 || rectangle.height <= 0) {
            throw new IllegalArgumentException("Rectangle width and height must be > 0");
        }
        return;
    }
    
    private static void checkScreenCaptureAllowed() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.AWT.READ_DISPLAY_PIXELS_PERMISSION);
        }
    }
    
    private void afterEvent() {
        this.autoWaitForIdle();
        this.autoDelay();
    }
    
    public synchronized boolean isAutoWaitForIdle() {
        return this.isAutoWaitForIdle;
    }
    
    public synchronized void setAutoWaitForIdle(boolean isAutoWaitForIdle) {
        this.isAutoWaitForIdle = isAutoWaitForIdle;
    }
    
    private void autoWaitForIdle() {
        if (this.isAutoWaitForIdle) {
            this.waitForIdle();
        }
    }
    
    public synchronized int getAutoDelay() {
        return this.autoDelay;
    }
    
    public synchronized void setAutoDelay(int autoDelay) {
        this.checkDelayArgument(autoDelay);
        this.autoDelay = autoDelay;
    }
    
    private void autoDelay() {
        this.delay(this.autoDelay);
    }
    
    public synchronized void delay(int n) {
        this.checkDelayArgument(n);
        try {
            Thread.sleep((long)n);
        }
        catch (InterruptedException ex) {
            ex.printStackTrace();
        }
    }
    
    private void checkDelayArgument(int n) {
        if (n < 0 || n > 60000) {
            throw new IllegalArgumentException("Delay must be to 0 to 60,000ms");
        }
        return;
    }
    
    public synchronized void waitForIdle() {
        this.checkNotDispatchThread();
        try {
            SunToolkit.flushPendingEvents();
            EventQueue.invokeAndWait(new Runnable() {
                @Override
                public void run() {
                }
            });
        }
        catch (InterruptedException ex) {
            System.err.println("Robot.waitForIdle, non-fatal exception caught:");
            ex.printStackTrace();
        }
        catch (InvocationTargetException ex2) {
            System.err.println("Robot.waitForIdle, non-fatal exception caught:");
            ex2.printStackTrace();
        }
    }
    
    private void checkNotDispatchThread() {
        if (EventQueue.isDispatchThread()) {
            throw new IllegalThreadStateException("Cannot call method from the event dispatcher thread");
        }
        return;
    }
    
    @Override
    public synchronized String toString() {
        return this.getClass().getName() + "[ " + ("autoDelay = " + this.getAutoDelay() + ", autoWaitForIdle = " + this.isAutoWaitForIdle()) + " ]";
    }
    
    static {
        Robot.LEGAL_BUTTON_MASK = 0;
    }
    
    static class RobotDisposer implements DisposerRecord
    {
        private final RobotPeer peer;
        
        public RobotDisposer(RobotPeer peer) {
            this.peer = peer;
        }
        
        @Override
        public void dispose() {
            if (this.peer != null) {
                this.peer.dispose();
            }
        }
    }
}
